#!/usr/bin/env python

# NOTE: Adding user only works with basic auth
# https://grafana.com/docs/http_api/admin/#permissions

import argparse
from collections import defaultdict
import json

import requests


class Grafana:
    def __init__(self, url, user=None, password=None, token=None):
        self.url = url
        self.user = user
        self.password = password

        headers = {'content-type': 'application/json'}

        if token is not None:
            headers.update({'Authorization': 'Bearer {}'.format(token)})
            self.kwargs = {'headers': headers}
        else:
            self.kwargs = {
                'auth': (user, password),
                'headers': headers
            }

    def _create(self, type_of, data, extra_kwargs=None):
        extra_kwargs = extra_kwargs or {}
        extra_kwargs.update(self.kwargs)
        r = requests.post('{}/api/{}s'.format(self.url, type_of),
                          data=json.dumps(data),
                          **extra_kwargs)
        return r.json()

    def _delete(self, type_of, id_of):
        r = requests.delete('{}/api/{}s/{}'.format(self.url, type_of, id_of),
                            **self.kwargs)
        return r.json()

    def _get(self, type_of, id_of):
        r = requests.get('{}/api/{}s/{}'.format(self.url, type_of, id_of),
                         **self.kwargs)
        return r.json()

    def _get_all(self, type_of):
        r = requests.get('{}/api/{}s/search'.format(self.url, type_of),
                         **self.kwargs)
        return r.json()

    def create_user(self, user):
        return self._create('admin/user', user)

    def create_team(self, team):
        return self._create('team', team)

    def delete_user(self, user_id):
        return self._delete('admin/user', user_id)

    def delete_team(self, team_id):
        return self._delete('team', team_id)

    def get_user(self, user):
        return self._get('user', user)

    def get_team(self, team):
        return self._get('team', team)

    def get_all_users(self):
        return self._get_all('user')['users']

    def get_all_teams(self):
        return self._get_all('team')['teams']

    def create_member(self, team_id, user_id):
        return self._create('teams/{}/member'.format(team_id),
                            {'userId': user_id})
        # r = requests.post('{}/api/teams/{}/members'.format(self.url, team_id),
        #                   data=json.dumps({'userId': user_id}),
        #                   auth=(self.user, self.password),
        #                   headers={'content-type': 'application/json'})
        # return r.json()

    def delete_memeber(self, team_id, user_id):
        return self._delete('teams/{}/member'.format(team_id), user_id)

        # r = requests.delete('{}/api/teams/{}/members/{}'.format(
        #                         self.url, team_id, user_id),
        #                     auth=(self.user, self.password),
        #                     headers={'content-type': 'application/json'})
        # return r.json()

    def get_all_members(self, team_id):
        r = requests.get('{}/api/teams/{}/members'.format(self.url, team_id),
                         **self.kwargs)
        return r.json()


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--dryrun', action='store_true')
    parser.add_argument('--url')
    parser.add_argument('--user')
    parser.add_argument('--password')
    parser.add_argument('--token')
    parser.add_argument('--teams', default='{}')
    parser.add_argument('--users', default='{}')
    return parser.parse_args()


def main():
    args = parse_args()
    gr_cli = Grafana(args.url, user=args.user, password=args.password,
                     token=args.token)

    current_teams = json.loads(args.teams)
    current_users = json.loads(args.users)

    if args.dryrun:
        print('Dry run. None of following actions will be applied.')

    # create
    for name, team in current_teams.items():
        team.update({'name': name})

        print('Creating team', team)
        if not args.dryrun:
            print(gr_cli.create_team(team))

    for login, user in current_users.items():
        if user.get('teams') is not None:
            user.pop('teams')

        user.update({'login': login})
        user.update({'name': login})

        print('Creating user', user)
        if not args.dryrun:
            print(gr_cli.create_user(user))

    team_map = {i['name']: i['id'] for i in gr_cli.get_all_teams()}
    user_map = {i['login']: i for i in gr_cli.get_all_users()}

    # delete
    teams_to_remove = {t_id for name, t_id in team_map.items()
                       if name not in current_teams}

    # prevent removing Admins
    users_to_remove = {user['id'] for login, user in user_map.items()
                       if not (login in current_users or user['isAdmin'])}

    print('Removing teams', teams_to_remove)
    print('Removing users', users_to_remove)

    if args.dryrun:
        return

    for t_id in teams_to_remove:
        gr_cli.delete_team(t_id)

    for u_id in users_to_remove:
        gr_cli.delete_user(u_id)

    members_map = defaultdict(set)
    # add members
    for login, user in current_users.items():
        if user.get('teams') is None:
            continue

        for team in user['teams']:
            members_map[team].add(user_map[login])
            print(gr_cli.create_member(team_map[team], user_map[login]))

    # remove members
    for team in current_teams:
        members = gr_cli.get_all_members(team_map[team])
        print(members)
        members = {m['userId'] for m in members}

        members_to_remove = members.difference(members_map[team])

        for u_id in members_to_remove:
            print(gr_cli.delete_memeber(team_map[team], u_id))


if __name__ == '__main__':
    main()
